'''
	Threading sync utilities
'''
import threading


class IndexGenerator(object):
	def __init__(self, maxvalue=None):
		self._lock = threading.RLock()
		self._next = 0
		self._entries = {}

		self._maxvalue = maxvalue

	def get(self):
		with self._lock:
			if self._maxvalue is not None and self._next >= self._maxvalue:
				return None

			index = self._next
			self.insert(index)

			return index

	def insert(self, index):
		with self._lock:
			if index in self._entries:
				# Vacant slot
				self._next = self._entries[index]
			else:
				self._next += 1

	def remove(self, index):
		with self._lock:
			self._entries[index] = self._next
			self._next = index


class AsyncValue(object):
	def __init__(self):
		self.value = None
		self.completed = threading.Event()

	def set(self, value):
		self.value = value
		self.completed.set()

	def get(self):
		while not self.completed.is_set():
			self.completed.wait()
		return self.value
